@@ -1,6 +1,6 @@
 #############################################################################
 #  
-#	$Id: core.py,v 2.68 2004/05/10 20:30:37 irmen Exp $
+#	$Id: core.py,v 2.81 2005/05/26 19:40:11 irmen Exp $
 #	Pyro Core Library
 #
 #	This is part of "Pyro" - Python Remote Objects
@@ -8,7 +8,7 @@
 #
 #############################################################################
 
-import sys, time, sre, traceback, os
+import sys, time, sre, os, weakref
 import Pyro
 import util, protocol, constants
 from errors import *
@@ -53,10 +53,13 @@ class ObjBase:
 		# This will usually introduce a cyclic reference between the
 		# object and the daemon. Use a weak ref if available.
 		# NOTE: if you correctly clean up the object (that is, disconnect it from the daemon)
-        # the cyclic reference is cleared correctly, and no problem occurs.
+		# the cyclic reference is cleared correctly, and no problem occurs.
 		# NOTE: you have to make sure your original daemon object doesn't get garbage collected
 		# if you still want to use the objects! You have to keep a ref. to the daemon somewhere.
-		self.daemon=util.weakref(daemon)
+		if daemon:
+			self.daemon=weakref.proxy(daemon)
+		else:
+			self.daemon=None
 	def setCodeValidator(self, v):
 		if not callable(v):
 			raise TypeError("codevalidator must be a callable object")
@@ -145,8 +148,10 @@ class ObjBase:
 			else:
 				# read bytecode from the client
 				code=marshal.loads(module[8:])
-			# finally, execute the module code in the right module.	A
+			# finally, execute the module code in the right module.
 			exec code in mod.__dict__
+			# store the bytecode for possible later reference
+			mod.__dict__['_PYRO_bytecode'] = module
 		else:
 			Log.warn('ObjBase','attempt to supply code denied: ',name,'from',str(sourceaddr))
 			raise PyroError('attempt to supply code denied')
@@ -333,7 +338,7 @@ def processStringURI(URI):
 		import naming
 		protocol=x.group('protocol')
 		if protocol=="PYRONAMESSL":
-			raise ProtocolError("NOT SUPPORTED YET: "+protocol) # XXX
+			raise ProtocolError("NOT SUPPORTED YET: "+protocol) # XXX obviously, this should be implemented
 		hostname=x.group('hostname') or x.group('onlyhostname')
 		port=x.group('port')
 		name=x.group('name')
@@ -377,6 +382,17 @@ def getProxyForURI(URI):
 def getAttrProxyForURI(URI):
 	return DynamicProxyWithAttrs(URI)
 
+class _RemoteMethod:
+	# method call abstraction, adapted from Python's xmlrpclib
+	# it would be rather easy to add nested method calls, but
+	# that is not compatible with the way that Pyro's method
+	# calls are defined to work ( no nested calls ) 
+	def __init__(self, send, name):
+		self.__send = send
+		self.__name = name
+	def __call__(self, *args, **kwargs):
+		return self.__send(self.__name, args, kwargs)
+
 class DynamicProxy:
 	def __init__(self, URI):
 		_checkInit() # init required
@@ -387,7 +403,6 @@ class DynamicProxy:
 		# Delay adapter binding to enable transporting of proxies.
 		# We just create an adapter, and don't connect it...
 		self.adapter = protocol.getProtocolAdapter(self.URI.protocol)
-		self._name=[]
 		# ---- don't forget to register local vars with DynamicProxyWithAttrs, see below
 	def __del__(self):
 		if 'adapter' in self.__dict__.keys():
@@ -409,10 +424,9 @@ class DynamicProxy:
 		c=DynamicProxy(self.URI)
 		return c
 	def __getattr__(self, name):
-		if name!="__getinitargs__":			# allows it to be safely pickled
-			self._name.append(name)
-			return self.__invokePYRO__
-		raise AttributeError()
+		if name=="__getinitargs__":			# allows it to be safely pickled
+			raise AttributeError()
+		return _RemoteMethod(self._invokePYRO, name)
 	def __repr__(self):
 		return "<"+self.__class__.__name__+" for "+str(self.URI)+">"
 	def __str__(self):
@@ -420,28 +434,24 @@ class DynamicProxy:
 	def __hash__(self):
 		# makes it possible to use this class as a key in a dict
 		return hash(self.objectID)
-	def __lt__(self,other):
-		# makes it possible to compare two proxies using objectID
-		return self.objectID<other.objectID
-	def __gt__(self,other):
-		# makes it possible to compare two proxies using objectID
-		return self.objectID>other.objectID
 	def __eq__(self,other):
 		# makes it possible to compare two proxies using objectID
-		return self.objectID==other.objectID
-	def __cmp__(self,other):
+		return hasattr(other,"objectID") and self.objectID==other.objectID
+	def __ne__(self,other):
 		# makes it possible to compare two proxies using objectID
-		return cmp(self.objectID,other.objectID)
+		return not hasattr(other,"objectID") or self.objectID!=other.objectID
 	def __nonzero__(self):
 		return 1
-	def __coerce__(self,other):  # XXX python 2.0.x calls this sometimes
-		return (self,other)  # yep, this works! weird huh?
+	def __coerce__(self,other):
+		# makes it possible to compare two proxies using objectID (cmp)
+		if hasattr(other,"objectID"):
+			return (self.objectID, other.objectID)
+		return None
 
-	def __invokePYRO__(self, *vargs, **kargs):
+	def _invokePYRO(self, name, vargs, kargs):
 		if not self.adapter.connected():
 			self.adapter.bindToURI(self.URI)
-		return self.adapter.remoteInvocation(self._name.pop(),
-						constants.RIF_VarargsAndKeywords, vargs, kargs)
+		return self.adapter.remoteInvocation(name, constants.RIF_VarargsAndKeywords, vargs, kargs)
 
 	# Pickling support, otherwise pickle uses __getattr__:
 	def __getstate__(self):
@@ -464,9 +474,10 @@ class DynamicProxyWithAttrs(DynamicProxy):
 		self._attr_cache = {}
 		DynamicProxy.__init__(self, URI)
 	def _r_ga(self, attr, value=0):
-		if value: self._name.append("_r_ga")
-		else: self._name.append("_r_ha")
-		return self.__invokePYRO__(attr)
+		if value:
+			return _RemoteMethod(self._invokePYRO, "_r_ga") (attr)  # getattr
+		else:
+			return _RemoteMethod(self._invokePYRO, "_r_ha") (attr)  # hasattr
 	def findattr(self, attr):
 		if attr in self._attr_cache.keys():
 			return self._attr_cache[attr]
@@ -474,25 +485,22 @@ class DynamicProxyWithAttrs(DynamicProxy):
 		self._attr_cache[attr] = self._r_ga(attr)
 		return self._attr_cache[attr]
 	def __copy__(self):		# create copy of current proxy object
-		c=DynamicProxyWithAttrs(self.URI)
-		return c
+		return DynamicProxyWithAttrs(self.URI)
 	def __setattr__(self, attr, value):
 		if attr in self.__dict__["_local_attrs"]:
 			self.__dict__[attr]=value
 		else:
 			result = self.findattr(attr)
 			if result==2: # attribute
-				self._name.append("_r_sa")
-				return self.__invokePYRO__(attr,value)
+				return _RemoteMethod(self._invokePYRO, "_r_sa") (attr,value)
 			else:
 				raise AttributeError('not an attribute')
 	def __getattr__(self, attr):
 		# allows it to be safely pickled
-		if attr not in ("__getinitargs__", "__hash__","__cmp__","__eq__"):
+		if attr not in ("__getinitargs__", "__hash__","__eq__","__ne__"):
 			result=self.findattr(attr)
 			if result==1: # method
-				self._name.append(attr)
-				return self.__invokePYRO__
+				return _RemoteMethod(self._invokePYRO, attr)
 			elif result:
 				return self._r_ga(attr, 1)
 		raise AttributeError
@@ -511,8 +519,7 @@ class DynamicProxyWithAttrs(DynamicProxy):
 class DaemonServant(ObjBase):
 	def __init__(self, daemon):
 		ObjBase.__init__(self)
-		# XXX next stmt could be a cyclic reference if weakrefs are not implemented :-(
-		self.daemon=util.weakref(daemon)
+		self.daemon=weakref.proxy(daemon)
 	def getRegistered(self):
 		return self.daemon.getRegistered()
 	def ResolvePYROLOC(self, name):
@@ -555,10 +562,16 @@ class Daemon(protocol.TCPServer, ObjBase):
 		Log.error('Daemon','Couldn\'t start Pyro daemon: ' +str(errormsg))
 		raise DaemonError('Couldn\'t start Pyro daemon: ' +str(errormsg))
 	
-	def __del__(self):
+	# to be called to stop all connections and shut down.
+	def shutdown(self, disconnect=False):
+		protocol.TCPServer.shutdown(self)
+		if disconnect:
+			self.__disconnectObjects()
+	def __disconnectObjects(self):
 		# server shutting down, unregister all known objects in the NS
 		if self.NameServer and constants:
-			del self.implementations[constants.INTERNAL_DAEMON_GUID]
+			if constants.INTERNAL_DAEMON_GUID in self.implementations:
+				del self.implementations[constants.INTERNAL_DAEMON_GUID]
 			if self.implementations:
 				Log.warn('Daemon','Shutting down but there are still',len(self.implementations),'objects connected - disconnecting them')
 			for guid in self.implementations.keys():
@@ -569,6 +582,11 @@ class Daemon(protocol.TCPServer, ObjBase):
 							self.NameServer.unregister(name)
 						except Exception,x:
 							Log.warn('Daemon','Error while unregistering object during shutdown:',x)
+			self.implementations={}
+		
+
+	def __del__(self):
+		self.__disconnectObjects() # unregister objects
 		if hasattr(self,'adapter'):
 			del self.adapter
 		if protocol: protocol.TCPServer.__del__(self)
@@ -598,7 +616,6 @@ class Daemon(protocol.TCPServer, ObjBase):
 		msg="daemon bound on hostname that resolves to loopback address 127.0.0.1"
 		Log.warn("Daemon",msg)
 		Log.warn("Daemon","hostname="+self.hostname)
-		print "\nWARNING: "+msg+"\n"
 		return msg
 	
 	def useNameServer(self,NS):
@@ -713,12 +730,12 @@ class Daemon(protocol.TCPServer, ObjBase):
 			(exc_type, exc_value, exc_trb) = sys.exc_info()
 			if exc_type==ProtocolError:
 				# Problem with the communication protocol, shut down the connection
-				# XXX is this wat we want???
+				# XXX is shutting down what we want???
 				Log.error('Daemon','protocol error occured:',exc_value)
 				Log.error('Daemon','Due to network error: shutting down connection with',conn)
 				self.removeConnection(conn)
 			else:
-				exclist = Pyro.util.formatTraceback(exc_type, exc_value, exc_trb)
+				exclist = util.formatTraceback(exc_type, exc_value, exc_trb)
 				out =''.join(exclist)
 				Log.warn('Daemon', 'Exception during processing of request from',
 					conn,' type',exc_type,
@@ -751,7 +768,7 @@ class Daemon(protocol.TCPServer, ObjBase):
 		for o in self.implementations.keys():
 			if self.implementations[o][1]==name:
 				return o
-		raise NamingError('no object found by this name')
+		raise NamingError('no object found by this name',name)
 
 
 #############################################################################
