@@ -1,6 +1,6 @@
 #############################################################################
 #
-#	$Id: naming.py,v 2.44 2004/02/05 00:42:33 irmen Exp $
+#	$Id: naming.py,v 2.54 2005/10/06 20:00:29 irmen Exp $
 #	Pyro Name Server
 #
 #	This is part of "Pyro" - Python Remote Objects
@@ -8,7 +8,7 @@
 #
 #############################################################################
 
-import sys, os, socket, string, time
+import sys, os, socket, time
 import dircache, shutil, SocketServer
 import Pyro
 import util, errors, core, protocol, constants
@@ -28,6 +28,9 @@ else:
 	errno.ENOTDIR=-1
 	
 
+NS_SYSCMD_LOCATION='location'
+NS_SYSCMD_SHUTDOWN='shutdown'
+
 Log = util.Log
 
 #############################################################################
@@ -47,72 +50,92 @@ class NameServerLocator:
 
 	def sendSysCommand(self,request,host=None,port=None,trace=0,logerrors=1):
 		try:
+			# Try the 'first' name server.
+			# Note that if no host is specified, a broadcast is used,
+			# and that one is sent to both name servers in parallel.
 			return self.__sendSysCommand(request, host, port, trace, logerrors, Pyro.constants.NSROLE_PRIMARY)
 		except KeyboardInterrupt:
 			raise
-		except Exception,x:
+		except Exception:
 			if not port:
 				# the 'first' name server failed, try the second
 				result=self.__sendSysCommand(request, host, port, trace, logerrors, Pyro.constants.NSROLE_SECONDARY)
 				# found the second!
-				# switch config for first and second
+				# switch config for first and second so that the second one (which we found) will now be tried first
 				Pyro.config.PYRO_NS2_HOSTNAME, Pyro.config.PYRO_NS_HOSTNAME = Pyro.config.PYRO_NS_HOSTNAME, Pyro.config.PYRO_NS2_HOSTNAME
 				Pyro.config.PYRO_NS2_PORT, Pyro.config.PYRO_NS_PORT = Pyro.config.PYRO_NS_PORT, Pyro.config.PYRO_NS2_PORT
 				Pyro.config.PYRO_NS2_BC_PORT, Pyro.config.PYRO_NS_BC_PORT = Pyro.config.PYRO_NS_BC_PORT, Pyro.config.PYRO_NS2_BC_PORT
 				return result
 			else:
-				raise x
+				raise
 
 	def __sendSysCommand(self,request,host=None,port=None,trace=0,logerrors=1,role=Pyro.constants.NSROLE_PRIMARY):
 		HPB={Pyro.constants.NSROLE_PRIMARY: (Pyro.config.PYRO_NS_HOSTNAME, Pyro.config.PYRO_NS_PORT, Pyro.config.PYRO_NS_BC_PORT),
 		     Pyro.constants.NSROLE_SECONDARY: (Pyro.config.PYRO_NS2_HOSTNAME, Pyro.config.PYRO_NS2_PORT, Pyro.config.PYRO_NS2_BC_PORT) }
 		if not host:
 			host=HPB[role][0]
-		if not port:
+		if port:
+			port1=port2=port
+		else:
 			if not host:
-				# select the default broadcast port
-				port = HPB[role][2]
+				# select the default broadcast ports
+				port1 = HPB[Pyro.constants.NSROLE_PRIMARY][2]
+				port2 = HPB[Pyro.constants.NSROLE_SECONDARY][2]
 			else:
-				# select the default port
+				# select the default port (normal)
 				port = HPB[role][1]
-
 		# We must discover the location of the name server.
 		# Pyro's NS can answer to broadcast requests.
 		try:
-			s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
-			if not host:
-				# No host specified. Use broadcast mechanism
-				if os.name=='java':
-					# Jython handling for broadcast addrs
-					destination = ('255.255.255.255', port)
-				else:
-					# regular python broadcast socket
-					destination = ('<broadcast>', port)
-				if hasattr(socket,'SO_BROADCAST'):
-					s.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
-			else:
+			if host:
 				# use direct lookup with PYROLOC: mechanism, no broadcast
 				if trace:
 					print 'Locator: contacting Pyro Name Server...'
 				uri=core.PyroURI(host,constants.NAMESERVER_NAME,port,'PYROLOC')
 				prox=core.getProxyForURI(uri)
 				prox._setIdentification(self.identification)
-				prox.ping()	# force resolving of PYROLOC: uri
-				return prox.URI # return resolved uri
+				if request==NS_SYSCMD_LOCATION:
+					prox.ping()	# force resolving of PYROLOC: uri
+					return prox.URI # return resolved uri
+				else:
+					try:
+						return prox._shutdown()
+					except Exception,x:
+						return "Error processing shutdown request. Try again. "+str(x)
+
+			# No host specified. Use broadcast mechanism
+			if os.name=='java':
+				# Jython handling for broadcast addrs
+				destination1 = ('255.255.255.255', port1)
+				destination2 = ('255.255.255.255', port2)
+			else:
+				# regular python broadcast socket
+				destination1 = ('<broadcast>', port1)
+				destination2 = ('<broadcast>', port2)
+			s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
+			if hasattr(socket,'SO_BROADCAST'):
+				s.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
 			
 			if trace:
 				print 'Locator: searching Pyro Name Server...'
-			for i in range(Pyro.config.PYRO_BC_RETRIES+1):
-				s.sendto(request, destination)  # send to Pyro NS's bc port
-				if os.name!='java':
-					ins,outs,exs = select.select([s],[],[s],Pyro.config.PYRO_BC_TIMEOUT)
-				else:
-					ins=[s]		# XXX preliminary Jython solution (no select)
-				if s in ins:
-					reply, fromaddr = s.recvfrom(1000)
-					return reply
-				if trace and i<Pyro.config.PYRO_BC_RETRIES:
-					print 'Locator: retry',i+1
+			try:
+				for i in range(Pyro.config.PYRO_BC_RETRIES+1):
+					# send request to both Pyro NS (if running in paired mode)
+					s.sendto(request, destination1)
+					if destination2!=destination1:
+						s.sendto(request, destination2)
+					if os.name!='java':
+						ins,outs,exs = select.select([s],[],[s],Pyro.config.PYRO_BC_TIMEOUT)
+					else:
+						ins=[s]		# XXX crappy Jython solution (no select)
+					if s in ins:
+						# return the info of the first NS that responded.
+						reply, fromaddr = s.recvfrom(1000)
+						return reply
+					if trace and i<Pyro.config.PYRO_BC_RETRIES:
+						print 'Locator: retry',i+1
+			finally:
+				s.close()
 		except socket.error,e:
 			if logerrors:
 				Log.error('NameServerLocator','network error:',e)
@@ -124,15 +147,15 @@ class NameServerLocator:
 	
 	def detectNS(self,host=None,port=None,trace=0):
 		# just try to detect an existing NS. Don't log errors.
-		return core.PyroURI(self.sendSysCommand('location',host,port,trace,0))
+		return core.PyroURI(self.sendSysCommand(NS_SYSCMD_LOCATION,host,port,trace,0))
 
 	def getNS(self,host=None,port=None,trace=0):
-		reply = self.sendSysCommand('location',host,port,trace)
+		reply = self.sendSysCommand(NS_SYSCMD_LOCATION,host,port,trace)
 		Log.msg('NameServerLocator','Name Server found:',reply)
 		ns=NameServerProxy(core.PyroURI(reply),self.identification)
 		ns._setIdentification(self.identification)
 		return ns
-
+		
 
 # NOTE: The NameServerProxy class below is hand crafted.
 # This is because we want to enforce the default group name on all name
@@ -196,6 +219,8 @@ class NameServerProxy:
 		return self.__remoteinvoc('getMeta',0,_expandName(name))
 	def fullName(self,name):
 		return _expandName(name)
+	def _shutdown(self):
+		return self.__remoteinvoc('_shutdown',0)
 	def _setSystemMeta(self, name, meta):
 		return self.__remoteinvoc('_setSystemMeta',0,_expandName(name),meta)
 	def _getSystemMeta(self, name):
@@ -211,6 +236,7 @@ class NameServerProxy:
 		return self.__dict__
 
 # Can be used to expand names to absolute names (NS proxy uses this)
+# but user code should use the fullName method of the proxy.
 def _expandName(name):
 	if Pyro.config.PYRO_NS_DEFAULTGROUP[0]!=':':
 		raise errors.NamingError('default group name is not absolute')
@@ -234,12 +260,17 @@ def _expandName(name):
 
 
 class NameServer(core.ObjBase):
-	def __init__(self, role=Pyro.constants.NSROLE_SINGLE):
+	def __init__(self, role=Pyro.constants.NSROLE_SINGLE, identification=None):
 		core.ObjBase.__init__(self)
 		self.root=NamedTree('<root>')
 		self.lock=util.getLockObject()
 		self.role=role
 		self.otherNS=None
+		if role in (Pyro.constants.NSROLE_PRIMARY, Pyro.constants.NSROLE_SECONDARY):
+			# for paired mode with identification, we need to remember the ident string
+			adapter=Pyro.protocol.getProtocolAdapter("PYRO")
+			adapter.setIdentification(identification)
+			self.identification=adapter.getIdentification() # grab the munged ident
 		# create default groups
 		self.createGroup(':'+'Pyro')
 		self.createGroup(Pyro.config.PYRO_NS_DEFAULTGROUP)
@@ -250,18 +281,26 @@ class NameServer(core.ObjBase):
 			
 	def _initialResyncWithTwin(self, twinProxy):
 		if twinProxy:
-			Log.msg("NameServer","Initial resync with other NS at",twinProxy.URI.address,"port",twinProxy.URI.port)
-			print "Resyncing with other NS at",twinProxy.URI.address,"port",twinProxy.URI.port
-			# keep old NS (self) registration
-			oldNSreg=self.resolve(Pyro.constants.NAMESERVER_NAME)
-			self.root=twinProxy._resync(NameServerProxy(self.getProxy().URI,noconnect=1))
-			# reset self registration
 			try:
-				self.unregister(Pyro.constants.NAMESERVER_NAME)
-			except:
-				pass
-			self.register(Pyro.constants.NAMESERVER_NAME,oldNSreg)
-			self.otherNS=twinProxy
+				Log.msg("NameServer","Initial resync with other NS at",twinProxy.URI.address,"port",twinProxy.URI.port)
+				print "Initial Resync with other NS at",twinProxy.URI.address,"port",twinProxy.URI.port
+				# keep old NS (self) registration
+				oldNSreg=self.resolve(Pyro.constants.NAMESERVER_NAME)
+				proxyForMe=NameServerProxy(self.getProxy().URI,noconnect=1)
+				proxyForMe.adapter.setIdentification(self.identification,munge=False) # set pre-munged ident
+				self.root=twinProxy._resync(proxyForMe)
+				# reset self registration
+				try:
+					self.unregister(Pyro.constants.NAMESERVER_NAME)
+				except:
+					pass
+				self.register(Pyro.constants.NAMESERVER_NAME,oldNSreg)
+				self.otherNS=twinProxy
+				Log.msg("NameServer","database sync complete.")
+				print "Database synchronized."
+			except Pyro.errors.NamingError,x:
+				print x
+				raise
 			
 	def _removeTwinNS(self):
 		self.otherNS=None
@@ -273,14 +312,14 @@ class NameServer(core.ObjBase):
 		try:
 			(group, name)=self.locateGrpAndName(name)
 			if isinstance(group,NameValue):
-				raise errors.NamingError('parent is no group')
+				raise errors.NamingError('parent is no group', group.name)
 			try:
 				group.newleaf(name,URI)
 				Log.msg('NameServer','registered',name,'with URI',str(URI))
-				self.__dosynccall("register",origname,URI)
+				self._dosynccall("register",origname,URI)
 			except KeyError:
 				Log.msg('NameServer','name already exists:',name)
-				raise errors.NamingError('name already exists')
+				raise errors.NamingError('name already exists',name)
 		finally:
 			self.lock.release()
 
@@ -292,12 +331,12 @@ class NameServer(core.ObjBase):
 			try:
 				group.cutleaf(name)
 				Log.msg('NameServer','unregistered',name)
-				self.__dosynccall("unregister",origname)
+				self._dosynccall("unregister",origname)
 			except KeyError:
-				raise errors.NamingError('name not found')
+				raise errors.NamingError('name not found',name)
 			except ValueError:
 				Log.msg('NameServer','attempt to remove a group:',name)
-				raise errors.NamingError('is a group')
+				raise errors.NamingError('is a group, not an object',name)
 		finally:
 			self.lock.release()
 
@@ -310,11 +349,11 @@ class NameServer(core.ObjBase):
 				return branch.value
 			else:
 				Log.msg('NameServer','attempt to resolve groupname:',name)
-				raise errors.NamingError('attempt to resolve groupname')
+				raise errors.NamingError('attempt to resolve groupname',name)
 		except KeyError:
-			raise errors.NamingError('name not found')
+			raise errors.NamingError('name not found',name)
 		except AttributeError:
-			raise errors.NamingError('group not found')
+			raise errors.NamingError('group not found',name)
 
 	def flatlist(self):
 		# return a dump
@@ -341,7 +380,7 @@ class NameServer(core.ObjBase):
 			self.otherNS=twinProxy
 			self.lock.acquire()
 			try:
-				return self.root
+				return self._getSyncDump()
 			finally:
 				self.lock.release()
 		else:
@@ -362,12 +401,12 @@ class NameServer(core.ObjBase):
 			try:
 				self._initialResyncWithTwin(self.otherNS)
 				return
-			except:
+			except Exception:
 				pass
 		raise errors.NamingError("cannot resync: twin NS is unknown or unreachable")
 
 	# local helper:
-	def __dosynccall(self, method, *args):
+	def _dosynccall(self, method, *args):
 		if self.role!=Pyro.constants.NSROLE_SINGLE and self.otherNS:
 			try:
 				self.otherNS._synccall(method, *args)
@@ -384,7 +423,7 @@ class NameServer(core.ObjBase):
 			try:
 				parent.newbranch(name)
 				Log.msg('NameServer','created group',groupname)
-				self.__dosynccall("createGroup",groupname)
+				self._dosynccall("createGroup",groupname)
 			except KeyError,x:
 				raise errors.NamingError(x)
 		finally:
@@ -401,12 +440,11 @@ class NameServer(core.ObjBase):
 			try:
 				parent.cutbranch(name)
 				Log.msg('NameServer','deleted group',name)
-				self.__dosynccall("deleteGroup",groupname)
+				self._dosynccall("deleteGroup",groupname)
 			except KeyError:
-				raise errors.NamingError('group not found')
+				raise errors.NamingError('group not found',groupname)
 			except ValueError:
-				Log.msg('NameServer','attempt to deleteGroup a non-group:',groupname)
-				raise errors.NamingError('is no group')
+				raise errors.NamingError('is no group',groupname)
 		finally:
 			self.lock.release()
 			
@@ -418,10 +456,9 @@ class NameServer(core.ObjBase):
 		try:
 			return self.getBranch(groupname).list()
 		except KeyError:
-			raise errors.NamingError('group not found')
+			raise errors.NamingError('group not found',groupname)
 		except AttributeError:
-			Log.msg('NameServer','attempt to list a non-group:',groupname)
-			raise errors.NamingError('is no group')
+			raise errors.NamingError('is no group',groupname)
 			
 	# --- meta info support
 	def setMeta(self, name, meta):
@@ -429,11 +466,11 @@ class NameServer(core.ObjBase):
 		try:
 			branch=self.getBranch(name)
 			branch.setMeta(meta)
-			self.__dosynccall("setMeta",name,meta)
+			self._dosynccall("setMeta",name,meta)
 		except KeyError:
-			raise errors.NamingError('name not found')
+			raise errors.NamingError('name not found',name)
 		except AttributeError:
-			raise errors.NamingError('group not found')
+			raise errors.NamingError('group not found',name)
 	
 	def getMeta(self, name):
 		name=self.validateName(name)
@@ -441,20 +478,20 @@ class NameServer(core.ObjBase):
 			branch=self.getBranch(name)
 			return branch.getMeta()
 		except KeyError:
-			raise errors.NamingError('name not found')
+			raise errors.NamingError('name not found',name)
 		except AttributeError:
-			raise errors.NamingError('group not found')
+			raise errors.NamingError('group not found',name)
 
 	def _setSystemMeta(self, name, meta):
 		name=self.validateName(name)
 		try:
 			branch=self.getBranch(name)
 			branch.setSystemMeta(meta)
-			self.__dosynccall("_setSystemMeta",name,meta)
+			self._dosynccall("_setSystemMeta",name,meta)
 		except KeyError:
-			raise errors.NamingError('name not found')
+			raise errors.NamingError('name not found',name)
 		except AttributeError:
-			raise errors.NamingError('group not found')
+			raise errors.NamingError('group not found',name)
 	
 	def _getSystemMeta(self, name):
 		name=self.validateName(name)
@@ -462,14 +499,23 @@ class NameServer(core.ObjBase):
 			branch=self.getBranch(name)
 			return branch.getSystemMeta()
 		except KeyError:
-			raise errors.NamingError('name not found')
+			raise errors.NamingError('name not found',name)
 		except AttributeError:
-			raise errors.NamingError('group not found')
+			raise errors.NamingError('group not found',name)
 
+    # --- shut down the server
+	def _shutdown(self):
+		print 'Name Server received shutdown request... will shutdown shortly...'
+		self.getDaemon().shutdown()
+		return "Will shut down shortly"
+        
 	# --- private methods follow
+	def _getSyncDump(self):
+		return self.root
+
 	def locateGrpAndName(self,name):
 		# ASSUME name is absolute (from root) (which is required here)
-		idx=string.rfind(name,'.')
+		idx=name.rfind('.')
 		if idx>=0:
 			# name is hierarchical
 			grpname=name[:idx]
@@ -477,7 +523,7 @@ class NameServer(core.ObjBase):
 			try:
 				return (self.getBranch(grpname), name)
 			except KeyError:
-				raise errors.NamingError('(parent)group not found')
+				raise errors.NamingError('(parent)group not found',grpname)
 		else:
 			# name is in root
 			return (self.root, name[1:])
@@ -486,19 +532,19 @@ class NameServer(core.ObjBase):
 		# ASSUME name is absolute (from root) (which is required here)
 		name=name[1:]
 		if name:
-			return reduce(lambda x,y: x[y], string.split(name,'.'), self.root)
+			return reduce(lambda x,y: x[y], name.split('.'), self.root)
 		else:
 			return self.root
 
 	def validateName(self,name):
 		if len(name)>=1 and name[0]==':':
-			if ('' not in string.split(name,'.')):
+			if ('' not in name.split('.')):
 				for i in name:
-					if ord(i)<33 or ord(i)>126:
-						raise errors.NamingError('invalid name')
+					if ord(i)<33 or ord(i)>126 or i=='\\':
+						raise errors.NamingError('invalid character(s) in name',name)
 				return name
 			else:
-				raise errors.NamingError('invalid name')
+				raise errors.NamingError('invalid name',name)
 		else:
 			# name is not absolute. Make it absolute.
 			return _expandName(name)
@@ -511,7 +557,7 @@ class NameServer(core.ObjBase):
 			u.reinitFromString(URI)
 			return u
 		except:
-			raise errors.NamingError('invalid URI')
+			raise errors.NamingError('invalid URI',URI)
 
 	def publishURI(self, uri, verbose=0):
 		# verbose is not used - always prints the uri.
@@ -547,10 +593,8 @@ class NameSpaceNode:
 		self.systemMeta = NameSpaceSystemMeta(time.time(), owner)
 		self.userMeta = meta
 	def getMeta(self):
-		# print "GETTING META OF",self,type(self)		# XXX
 		return self.userMeta
 	def getSystemMeta(self):
-		# print "GETTING SYSMETA OF",self,type(self)		# XXX
 		return self.systemMeta
 	def setMeta(self,meta):
 		self.userMeta=meta
@@ -571,13 +615,13 @@ class NamedTree(NameSpaceNode):
 		self.branches={}
 	def newbranch(self,name):
 		if name in self.branches.keys():
-			raise KeyError,'duplicate name'
+			raise KeyError,'name already exists'
 		t = NamedTree(name)
 		self.branches[name]=t
 		return t
 	def newleaf(self,name,value=None):
 		if name in self.branches.keys():
-			raise KeyError,'duplicate name'
+			raise KeyError,'name already exists'
 		l = NameValue(name,value)
 		self.branches[name]=l
 		return l
@@ -622,19 +666,26 @@ class NamedTree(NameSpaceNode):
 #
 #############################################################################
 
+_PNS_META_SUFFIX=".ns_meta"
+
 class PersistentNameServer(NameServer):
-	def __init__(self, dbdir=None, role=Pyro.constants.NSROLE_SINGLE):
-		# root is not a NamedTree but a directory
+	def __init__(self, dbdir=None, role=Pyro.constants.NSROLE_SINGLE, identification=None):
 		self.dbroot=os.path.join(Pyro.config.PYRO_STORAGE,dbdir or 'Pyro_NS_database')
+		self._initdb_1()
+		try:
+			NameServer.__init__(self, role=role, identification=identification)
+		except errors.NamingError:
+			pass
+		self._initdb_2()
+
+	def _initdb_1(self):
+		# root is not a NamedTree but a directory
 		try:
 			os.mkdir(self.dbroot)
 		except OSError,x:
 			if x.errno not in (errno.EEXIST, errno.EBUSY):
 				raise
-		try:
-			NameServer.__init__(self, role=role)
-		except errors.NamingError:
-			pass
+	def _initdb_2(self):
 		# make sure that the 2 initial groups (Pyro and Default) exist
 		try: self.createGroup(':'+'Pyro')
 		except errors.NamingError: pass
@@ -644,17 +695,64 @@ class PersistentNameServer(NameServer):
 	def getDBDir(self):
 		return self.dbroot
 
+	def _initialResyncWithTwin(self, twinProxy):
+		if twinProxy:
+			Log.msg("NameServer","Initial resync with other NS at",twinProxy.URI.address,"port",twinProxy.URI.port)
+			# keep old NS (self) registration
+			oldNSreg=self.resolve(Pyro.constants.NAMESERVER_NAME)
+			proxyForMe=NameServerProxy(self.getProxy().URI,noconnect=1)
+			proxyForMe.adapter.setIdentification(self.identification,munge=False) # set pre-munged ident
+			syncdump=twinProxy._resync(proxyForMe)
+			self.otherNS = None   # temporarily disable twin NS ref
+			# clear the old database
+			Log.msg("NameServer","erasing old database",self.dbroot)
+			shutil.rmtree(self.dbroot)
+			self._initdb_1()   # phase 2 (creation of default groups) is not needed here
+			Log.msg("NameServer","store sync database")
+			for group,smeta,umeta in syncdump[0]:
+				try:
+					if group!=':':
+						dirnam = self.translate(group)
+						os.mkdir(dirnam)
+					if smeta:
+						self._setSystemMeta(group,smeta)
+					if umeta:
+						self.setMeta(group,umeta)
+				except EnvironmentError,x:
+					Log.warn("NameServer","problem creating group",group,x)
+			for name,uri,smeta,umeta in syncdump[1]:
+				try:
+					origname,name=name,self.validateName(name)
+					fn=self.translate(name)
+					open(fn,'w').write(uri+'\n')
+					if smeta:
+						self._setSystemMeta(name,smeta)
+					if umeta:
+						self.setMeta(name,umeta)
+				except errors.NamingError,x:
+					Log.warn("NameServer","problem registering name",name,x)
+			# reset registration of self
+			try:
+				self.unregister(Pyro.constants.NAMESERVER_NAME)
+			except:
+				pass
+			self.register(Pyro.constants.NAMESERVER_NAME,oldNSreg)
+			self.otherNS=twinProxy
+			Log.msg("NameServer","database sync complete.")
+			print "Database synchronized."
+
 	def register(self,name,URI):
-		name=self.validateName(name)
+		origname,name=name,self.validateName(name)
 		URI=self.validateURI(URI)
-		file=self.translate(name)
+		fn=self.translate(name)
 		self.lock.acquire()
 		try:
-			if os.access(file,os.R_OK):
+			if os.access(fn,os.R_OK):
 				Log.msg('NameServer','name already exists:',name)
-				raise errors.NamingError('name already exists')
+				raise errors.NamingError('name already exists',name)
 			try:
-				open(file,'w').write(str(URI)+'\n')
+				open(fn,'w').write(str(URI)+'\n')
+				self._dosynccall("register",origname,URI)
 				Log.msg('NameServer','registered',name,'with URI',str(URI))
 			except IOError,x:
 				if x.errno==errno.ENOENT:
@@ -667,19 +765,20 @@ class PersistentNameServer(NameServer):
 			self.lock.release()
 
 	def unregister(self,name):
-		name=self.validateName(name)
-		file=self.translate(name)
+		origname,name=name,self.validateName(name)
+		fn=self.translate(name)
 		self.lock.acquire()
 		try:
 			try:
-				os.remove(file)
+				os.remove(fn)
+				self._dosynccall("unregister",origname)
 				Log.msg('NameServer','unregistered',name)
 			except OSError,x:
 				if x.errno==errno.ENOENT:
-					raise errors.NamingError('name not found')
+					raise errors.NamingError('name not found',name)
 				elif x.errno==errno.EISDIR:
 					Log.msg('NameServer','attempt to remove a group:',name)
-					raise errors.NamingError('is a group')
+					raise errors.NamingError('is a group, not an object',name)
 				else:
 					raise errors.NamingError(x)
 		finally:
@@ -688,17 +787,17 @@ class PersistentNameServer(NameServer):
 	def resolve(self,name):
 		# not thread-locked: higher performance and not necessary.
 		name=self.validateName(name)
-		file = self.translate(name)
+		fn = self.translate(name)
 		try:
 			u = core.PyroURI('')
-			u.reinitFromString(open(file).read())
+			u.reinitFromString(open(fn).read())
 			return u
 		except IOError,x:
 			if x.errno==errno.ENOENT:
-				raise errors.NamingError('name not found')
+				raise errors.NamingError('name not found',name)
 			elif x.errno==errno.EISDIR:
 				Log.msg('NameServer','attempt to resolve groupname:',name)
-				raise errors.NamingError('attempt to resolve groupname')
+				raise errors.NamingError('attempt to resolve groupname',name)
 			else:
 				raise errors.NamingError(x)
 
@@ -706,26 +805,27 @@ class PersistentNameServer(NameServer):
 		dbroot=self.translate(':')
 		self.lock.acquire()
 		try:
-			files=self._filelist(dbroot,dbroot)
-			list=[]
-			for f in files:
-				list.append((f, self.resolve(f)))
-			return list
+			flat=[]
+			for f in self._filelist(dbroot,dbroot):
+				f=self._unescapefilename(f)
+				flat.append((f, self.resolve(f)))
+			return flat
 		finally:
 			self.lock.release()
 
 	# --- hierarchical naming support
 	def createGroup(self,groupname):
 		groupname=self.validateName(groupname)
-		file = self.translate(groupname)
+		dirnam = self.translate(groupname)
 		self.lock.acquire()
 		try:
 			try:
-				os.mkdir(file)
+				os.mkdir(dirnam)
+				self._dosynccall("createGroup",groupname)
 				Log.msg('NameServer','created group',groupname)
 			except OSError,x:
 				if x.errno in (errno.EEXIST, errno.EBUSY):
-					raise errors.NamingError('duplicate name')
+					raise errors.NamingError('group already exists',groupname)
 				elif x.errno == errno.ENOENT:
 					raise errors.NamingError('(parent)group not found')
 				else:
@@ -738,20 +838,20 @@ class PersistentNameServer(NameServer):
 		if groupname==':':
 			Log.msg('NameServer','attempt to deleteGroup root group')
 			raise errors.NamingError('not allowed to delete root group')
-		file = self.translate(groupname)
+		dirnam = self.translate(groupname)
 		self.lock.acquire()
 		try:
-			if not os.access(file,os.R_OK):
-				raise errors.NamingError('group not found')
+			if not os.access(dirnam,os.R_OK):
+				raise errors.NamingError('group not found',groupname)
 			try:
-				shutil.rmtree(file)
+				shutil.rmtree(dirnam)
+				self._dosynccall("deleteGroup",groupname)
 				Log.msg('NameServer','deleted group',groupname)
 			except OSError,x:
 				if x.errno==errno.ENOENT:
-					raise errors.NamingError('group not found')
+					raise errors.NamingError('group not found',groupname)
 				elif x.errno==errno.ENOTDIR:
-					Log.msg('NameServer','attempt to deleteGroup a non-group:',groupname)
-					raise errors.NamingError('is no group')
+					raise errors.NamingError('is no group',groupname)
 				else:
 					raise errors.NamingError(x)
 		finally:
@@ -761,29 +861,74 @@ class PersistentNameServer(NameServer):
 		if not groupname:
 			groupname=':'
 		groupname=self.validateName(groupname)
-		dir=self.translate(groupname)
+		dirnam=self.translate(groupname)
 		self.lock.acquire()
 		try:
-			if os.access(dir,os.R_OK):
-				if os.path.isfile(dir):
-					Log.msg('NameServer','attempt to list a non-group:',groupname)
-					raise errors.NamingError('is no group')
+			if os.access(dirnam,os.R_OK):
+				if os.path.isfile(dirnam):
+					raise errors.NamingError('is no group',groupname)
 				else:
-					l = dircache.listdir(dir)
-					list = []
+					l = dircache.listdir(dirnam)
+					entries = []
 					for e in l:
-						if os.path.isdir(os.path.join(dir,e)):
-							list.append((e,0))		# dir has code 0
+						if e.endswith(_PNS_META_SUFFIX):
+							continue
 						else:
-							list.append((e,1))		# leaf has code 1
-					return list
-			raise errors.NamingError('group not found')
+							objname=self._unescapefilename(e)
+							if os.path.isdir(os.path.join(dirnam,e)):
+								entries.append((objname,0))		# dir has code 0
+							else:
+								entries.append((objname,1))		# leaf has code 1
+					return entries
+			raise errors.NamingError('group not found',groupname)
 		finally:
 			self.lock.release()
 
 
 	# --- private methods follow
+	
+	def _getSyncDump(self):
+		def visitor(arg,dirname,names):
+			shortdirname=dirname[len(self.dbroot)+len(os.path.sep):]
+			if shortdirname.endswith(_PNS_META_SUFFIX):
+				return
+			name = ':'+shortdirname.replace(os.path.sep,'.')
+			smeta=self._getSystemMeta(name)
+			umeta=self.getMeta(name)
+			arg[0].append( (name, smeta,umeta) )
+			for n in names:
+				if n.endswith(_PNS_META_SUFFIX):
+					continue
+				n=os.path.join(dirname,n)
+				if os.path.isfile(n):
+					v=open(n,'r').read().strip()
+					name=':'+(n[len(self.dbroot)+len(os.path.sep):]).replace(os.path.sep,'.')
+					smeta=self._getSystemMeta(name)
+					umeta=self.getMeta(name)
+					arg[1].append( (name, v, smeta,umeta) )
+		result=( [], [] )   # (groups, names)
+		os.path.walk(self.dbroot, visitor, result)
+		return result
 
+	def _unescapefilename(self, name):
+		parts=name.split('\\')
+		res=[parts[0]]
+		myappend=res.append
+		del parts[0]
+		for item in parts:
+			if item[1:2]:
+				try:
+					myappend(chr(int(item[:2], 16)) + item[2:])
+				except ValueError:
+					myappend('\\' + item)
+			else:
+				myappend('\\' + item)
+		return "".join(res)
+	def _escapefilename(self,name):
+		name=name.replace(os.path.sep,'\\%02x' % ord(os.path.sep)) # escape path separators in the name
+		name=name.replace(':','\\%02x' % ord(':')) # also get rid of any ':' 's
+		return name
+		
 	# recursive file listing, output is like "find <path> -type f"
 	# but using NS group separator chars
 	def _filelist(self,root,path):
@@ -794,10 +939,12 @@ class PersistentNameServer(NameServer):
 			
 		files=[]
 		for f in filez:
-			if path==root:
+			if f.endswith(_PNS_META_SUFFIX):
+				continue
+			elif path==root:
 				files.append(':'+f)
 			else:
-				p=string.replace(path[len(root):], os.sep, '.')
+				p=path[len(root):].replace(os.sep, '.')
 				files.append(':'+p+'.'+f)
 		for d in dirz:
 			files.extend(self._filelist(root,os.path.join(path,d)))
@@ -807,28 +954,33 @@ class PersistentNameServer(NameServer):
 	def translate(self,name):
 		if name[0]==':':
 			name=name[1:]
-		args=[self.dbroot]+string.split(name,'.')
+		name=self._escapefilename(name)
+		args=[self.dbroot]+name.split('.')
 		return os.path.join(*args)
 
 	def getBranch(self,name):
 		tr = self.translate(name)
 		if os.path.exists(tr):
-			return PersistentNameSpaceNode(filename=tr+".ns_meta")
+			return PersistentNameSpaceNode(filename=tr+_PNS_META_SUFFIX)
 		else:
-			raise errors.NamingError('name not found')
+			raise errors.NamingError('name not found',name)
 
+# XXX this is a bit of a hack. Only metadata is stored here,
+# and it's only used from getBranch, which in turn is only used
+# from the set/get meta functions.
 class PersistentNameSpaceNode(NameSpaceNode):
 	def __init__(self, filename, name=None, meta=None, owner=None):
 		NameSpaceNode.__init__(self, name, meta, owner)
 		self.filename=filename
 		if not name:
+			# print "INIT PERSISTENT NODE FROM FILE",self.filename
 			# init from file
 			try:
 				(sysmeta, usermeta)=util.getPickle().load(open(self.filename,"rb"))
 				NameSpaceNode.setSystemMeta(self, sysmeta)
 				NameSpaceNode.setMeta(self, usermeta)
-			except Exception,x:
-				pass # XXX just accept empty meta...
+			except Exception:
+				pass # just use empty meta...
 		else:
 			# write to file
 			self._writeToFile()
@@ -839,6 +991,7 @@ class PersistentNameSpaceNode(NameSpaceNode):
 		NameSpaceNode.setSystemMeta(self, meta)
 		self._writeToFile()
 	def _writeToFile(self):
+		# print "WRITE PERSISTENT NODE TO FILE",self.filename
 		util.getPickle().dump( (self.getSystemMeta(), self.getMeta()) , open(self.filename,"wb"), Pyro.config.PYRO_PICKLE_FORMAT)
 		
 
@@ -865,7 +1018,7 @@ class BroadcastServer(SocketServer.UDPServer):
 			try:
 				SocketServer.UDPServer.__init__(self, (location,port), bcRequestHandler)
 				return			# got it!
-			except socket.error,x:
+			except socket.error:
 				continue		# try the next port in the list
 		raise    # port range exhausted... re-raise the socket error.
 		
@@ -897,10 +1050,10 @@ class bcRequestHandler(SocketServer.BaseRequestHandler):
 		Log.msg('BroadcastServer','incoming request from',str(self.client_address[0]))
 		# request is a simple string
 		cmd = self.request[0]
-		if cmd=='location':
+		if cmd==NS_SYSCMD_LOCATION:
 			# somebody wants to know our location, give them our URI
 			self.request[1].sendto(self.server.nameServerURI,self.client_address)
-		elif cmd=='shutdown':
+		elif cmd==NS_SYSCMD_SHUTDOWN:
 			# we should die!?
 			if self.server.ignoreShutdown:
 				Log.msg('BroadcastServer','Shutdown ignored.')
@@ -919,9 +1072,9 @@ class BCReqValidator:
 	def __call__(self, req, addr):
 		(cmd,self.sock)=req
 		self.addr=addr
-		if cmd=='location':
+		if cmd==NS_SYSCMD_LOCATION:
 			return self.acceptLocationCmd()
-		elif cmd=='shutdown':
+		elif cmd==NS_SYSCMD_SHUTDOWN:
 			return self.acceptShutdownCmd()
 		else:
 			return 0
@@ -1011,11 +1164,11 @@ class NameServerStarter:
 			daemon.setNewConnectionValidator(Guards[0])
 
 		if persistent:
-			ns=PersistentNameServer(dbdir,role=role[0])
+			ns=PersistentNameServer(dbdir,role=role[0], identification=self.identification)
 			daemon.useNameServer(ns)
 			NS_URI=daemon.connectPersistent(ns,constants.NAMESERVER_NAME)
 		else:
-			ns=NameServer(role=role[0])
+			ns=NameServer(role=role[0], identification=self.identification)
 			daemon.useNameServer(ns)
 			NS_URI=daemon.connect(ns,constants.NAMESERVER_NAME)
 
@@ -1046,6 +1199,9 @@ class NameServerStarter:
 
 			print 'Name server listening on:',daemon.sock.getsockname()
 			print 'Broadcast server listening on:',self.bcserver.socket.getsockname()
+			message = daemon.validateHostnameAndIP()
+			if message:
+				print "\nWARNING:",message,"\n"
 
 		if Guards[0] or Guards[1]:
 			if verbose:
@@ -1142,18 +1298,20 @@ class NameServerStarter:
 						port=None
 					else:
 						port=int(port)
-					otherNS=NameServerLocator().getNS(host,port,trace=0)
+					otherNS=NameServerLocator(self.identification).getNS(host,port,trace=0)
 				else:
 					if otherNSuri:
-						otherNS=NameServerLocator().getNS(host=otherNSuri.address, port=otherNSuri.port, trace=0)
+						otherNS=NameServerLocator(self.identification).getNS(host=otherNSuri.address, port=otherNSuri.port, trace=0)
 					else:
 						if role[0]==Pyro.constants.NSROLE_PRIMARY:
 							port=Pyro.config.PYRO_NS2_BC_PORT
 						else:
 							port=Pyro.config.PYRO_NS_BC_PORT
-						otherNS=NameServerLocator().getNS(host=None,port=port,trace=0)
+						otherNS=NameServerLocator(self.identification).getNS(host=None,port=port,trace=0)
 				Log.msg("NameServerStarted","Found twin NS at",otherNS.URI)
 				return otherNS
+			except Pyro.errors.ConnectionDeniedError,x:
+				raise
 			except Exception,x:
 				print "WARNING: Cannot find twin NS yet: ",x
 				Log.msg("NameServerStarter","Cannot find twin NS yet:",x)
@@ -1241,7 +1399,7 @@ def main(argv):
 
 	Args.printIgnored()
 	if Args.args:
-		print 'Ignored arguments:',string.join(Args.args)
+		print 'Ignored arguments:', ' '.join(Args.args)
 
 	print '*** Pyro Name Server ***'
 	if ident:
