@@ -1,6 +1,6 @@
 #############################################################################
 #
-#	$Id: protocol.py,v 2.62 2004/05/10 20:30:37 irmen Exp $
+#	$Id: protocol.py,v 2.77 2005/05/26 21:25:17 irmen Exp $
 #	Pyro Protocol Adapters
 #
 #	This is part of "Pyro" - Python Remote Objects
@@ -8,7 +8,7 @@
 #
 #############################################################################
 
-import socket, struct, os, time, sys, md5
+import socket, struct, os, time, sys, md5, hmac
 import Pyro
 import util, constants
 
@@ -22,7 +22,6 @@ Log = util.Log
 
 	
 if util.supports_multithreading():
-	# XXX if using M2Crypto, should we use M2Crypto.threading???
 	from threading import Thread,currentThread
 	_has_threading = 1
 else:
@@ -56,6 +55,16 @@ def getIPAddress(host=None):
 	
 #------ Socket helper functions for sending and receiving data correctly.
 
+
+# process optional timeout on socket.
+# XXX replace this with python's native socket timeouts. M2Crypto (SSL) needs special care...
+def _sock_timeout(sock, timeout):
+	ssl_select_okay=not hasattr(sock,'pending') or sock.pending()==0
+	if timeout and ssl_select_okay:
+		r,w,e=select.select([],[sock],[],timeout)
+		if not w:
+			raise TimeoutError('connection timeout sending')
+
 # Receive a precise number of bytes from a socket. Raises the
 # ConnectionClosedError if  that number of bytes was not available.
 # (the connection has probably been closed then).
@@ -68,27 +77,51 @@ def sock_recvmsg(sock, size, timeout=0):
 		# when using SSL, other exceptions occur.
 		from M2Crypto.SSL import SSLError
 		try:
-			return _recv_msg(sock,size,timeout,1)
+			return _recv_msg(sock,size,timeout)
 		except SSLError:
 			raise ConnectionClosedError('connection lost')
 	else:
 		try:
-			return _recv_msg(sock,size,timeout,0)
+			return _recv_msg(sock,size,timeout)
 		except socket.error:
 			raise ConnectionClosedError('connection lost')
 
-def _recv_msg(sock,size,timeout,ssl):
+# select the optimal recv() implementation
+if hasattr(socket.socket, "recvall"):   # Irmen's custom socket module extension, see Python patch #1103213
+	def _recv_msg(sock,size,timeout):
+		_sock_timeout(sock,timeout)
+		chunk=sock.recvall(size)
+		if len(chunk)!=size:
+			err=ConnectionClosedError('connection lost')
+			err.partialMsg=chunk    # store the message that was received until now
+			raise err
+		return chunk
+elif hasattr(socket,"MSG_WAITALL"):
+	def _recv_msg(sock,size,timeout):
+		_sock_timeout(sock,timeout)
+		try:		
+			chunk=sock.recv(size, socket.MSG_WAITALL)   # receive all data in one call
+		except TypeError:
+			# XXX This is caused by a bug in the M2Crypto API, it doesn't like the MSG_WAITALL parameter
+			# Once the bug is fixed, the compat function can be rolled back into the _recv_msg below.
+			return __recv_msg_compat(sock,size,timeout)
+		else:
+			if len(chunk)!=size:
+				err=ConnectionClosedError('connection lost')
+				err.partialMsg=chunk  # store the message that was received until now
+				raise err
+			return chunk
+else:
+	def _recv_msg(sock,size,timeout):
+		_sock_timeout(sock, timeout)
+		return __recv_msg_compat(sock,size,timeout)
+
+def __recv_msg_compat(sock,size,timeout):   # compatibility implementation for non-MSG_WAITALL / M2Crypto
 	msglen=0
 	msglist=[]
+	# Receive chunks of max. 60kb size:
+	# (rather arbitrary limit, but it avoids memory/buffer problems on certain OSes -- VAX, Windows)
 	while msglen<size:
-		# XXX select must not be called when SSL socket has bytes pending
-		ssl_select_okay=not ssl or (ssl and sock.pending()==0)
-		if timeout and ssl_select_okay:
-			r,w,e=select.select([sock],[],[],timeout)
-			if not r:
-				raise TimeoutError('connection timeout receiving')
-		# Receive a chunk of max. 60kb size:
-		# (rather arbitrary limit, but it avoids memory/buffer problems on certain OSes)
 		chunk=sock.recv(min(60000,size-msglen))
 		if not chunk:
 			err = ConnectionClosedError('connection lost')
@@ -105,24 +138,9 @@ def _recv_msg(sock,size,timeout,ssl):
 # bytes in one call, for instance, when network load is high.
 
 def sock_sendmsg(sock,msg,timeout=0):
-	size=len(msg)
-	total=0
 	try:
-		# XXX select must not be called when SSL socket has bytes pending
-		ssl_select_okay=not hasattr(sock,'pending') or sock.pending()==0
-		if timeout and ssl_select_okay:
-			r,w,e=select.select([],[sock],[],timeout)
-			if not w:
-				raise TimeoutError('connection timeout sending')
-		sent=sock.send(msg)
-		total+=sent
-		if sent==0:
-			raise ConnectionClosedError('connection lost')
-		while total<size:
-			sent=sock.send(msg[total:])
-			if sent==0:
-				raise ConnectionClosedError('connection lost')
-			total+=sent
+		_sock_timeout(sock,timeout)
+		sock.sendall(msg)
 	except socket.error:
 		raise ConnectionClosedError('connection lost')
 
@@ -240,8 +258,7 @@ class PYROAdapter:
 	def _sendConnect(self, sock, ident):
 		body=self.connectMSG+ident
 		sock_sendmsg(sock, self.createMsg(body), self.timeout)
-		result= sock_recvmsg(sock, len(self.acceptMSG),self.timeout)
-		return result
+		return sock_recvmsg(sock, len(self.acceptMSG),self.timeout)
 
 	def release(self,nolog=0):
 		if hasattr(self,'conn'):
@@ -265,7 +282,7 @@ class PYROAdapter:
 				t+=1
 				if t<tries:
 					time.sleep(wait)
-		raise ConnectionClosedError('connection lost')
+		raise TimeoutError('connection lost')
 		
 	def createMsg(self, body, replyflags=0):
 		pflgs=replyflags
@@ -289,11 +306,16 @@ class PYROAdapter:
 		self.onewayMethods.extend(methods)
 	def setTimeout(self, timeout):
 		self.timeout=timeout
-	def setIdentification(self, ident):
+	def setIdentification(self, ident, munge=True):
 		if ident:
-			self.ident=self.newConnValidator.mungeIdent(ident)   # don't store ident itself. 
+			if munge:
+				self.ident=self.newConnValidator.mungeIdent(ident)   # don't store ident itself. 
+			else:
+				self.ident=ident # per-munged ident string
 		else:
 			self.ident=''
+	def getIdentification(self):
+		return self.ident
 
 	# Retrieve code from the remote peer. Works recursively.
 	def _retrieveCode(self, mname, level):
@@ -334,7 +356,7 @@ class PYROAdapter:
 				try:
 					exec code in mod.__dict__
 					loaded = 1
-				except ImportError, x:
+				except ImportError:
 					mname = importer.name
  
 					if importer is not None:
@@ -364,7 +386,7 @@ class PYROAdapter:
 			return None		# no answer required, return immediately
 		ver,answer,pflags = self.receiveMsg(self.conn,1)
 		if answer is None:
-			raise ProtocolError('incorrect answer received') #XXX
+			raise ProtocolError('incorrect answer received')
 
 		# Try to get the answer from the server.
 		# If there are import problems, try to get those modules from
@@ -383,7 +405,7 @@ class PYROAdapter:
 					try:
 						answer = pickle.loads(answer)
 						loaded = 1
-					except ImportError, x:
+					except ImportError:
 						mname = importer.name
 						if importer is not None:
 							__builtin__.__import__ = importer.orig_import
@@ -408,20 +430,28 @@ class PYROAdapter:
 						Log.error('PYROAdapter','Server wanted a non-existing module:',mname)
 						raise PyroError('Server wanted a non-existing module',mname)
 					m=eval('importmodule.'+mname)
-					# try to load the module's compiled source, or the real .py source if that fails.
-					(filebase,ext)=os.path.splitext(m.__file__)
-					if ext.startswith(".PY"):
-						exts = ( ".PYO", ".PYC", ".PY" )	# uppercase
+					bytecode=None
+					if hasattr(m,"_PYRO_bytecode"):
+						bytecode=m._PYRO_bytecode
 					else:
-						exts = ( ".pyo", ".pyc", ".py" )	# lowercase
-					for ext in exts:
-						try:
-							m=open(filebase+ext, "rb").read()
-							self.remoteInvocation("remote_supply_code",0,mname, m, self.conn.sock.getsockname())
-							# retry the method invocation
-							return self.remoteInvocation(* (method, flags)+args)
-						except:
-							pass
+						# try to load the module's compiled source, or the real .py source if that fails.
+						if not hasattr(m,"__file__"):
+							raise PyroError("cannot read module source code",mname)
+						(filebase,ext)=os.path.splitext(m.__file__)
+						if ext.startswith(".PY"):
+							exts = ( ".PYO", ".PYC", ".PY" )	# uppercase
+						else:
+							exts = ( ".pyo", ".pyc", ".py" )	# lowercase
+						for ext in exts:
+							try:
+								bytecode=open(filebase+ext, "rb").read()
+								break
+							except EnvironmentError:
+								pass
+					if bytecode:
+						self.remoteInvocation("remote_supply_code",0,mname, bytecode, self.conn.sock.getsockname())
+						# retry the method invocation
+						return self.remoteInvocation(* (method, flags)+args)
 					Log.error("PYROAdapter","cannot read module source code for module:", mname)
 					raise PyroError("cannot read module source code",mname)
 				finally:
@@ -434,7 +464,7 @@ class PYROAdapter:
 	# (private) receives a socket message, returns: (protocolver, message, protocolflags)
 	def receiveMsg(self,conn,noReply=0):
 		msg=sock_recvmsg(conn.sock, self.headerSize, self.timeout)
-		(id, ver, hsiz, bsiz, pflags, crc) = struct.unpack(self.headerFmt,msg)
+		(hid, ver, hsiz, bsiz, pflags, crc) = struct.unpack(self.headerFmt,msg)
 		# store in the connection what pickle method this is
 		if pflags&PFLG_XMLPICKLE_GNOSIS:
 			conn.pflags|=PFLG_XMLPICKLE_GNOSIS
@@ -448,9 +478,9 @@ class PYROAdapter:
 				# try to report error to client, but most likely the connection will terminate:
 				self.returnException(conn, ProtocolError('incompatible protocol version'))
 				return ver,None,pflags
-		if id!=self.headerID or hsiz!=self.headerSize:
+		if hid!=self.headerID or hsiz!=self.headerSize:
 			Log.error('PYROAdapter','invalid header')
-			Log.error('PYROAdapter','INVALID HEADER DETAILS: ',conn,( id, ver, hsiz, bsiz,pflags)) # XXX
+			Log.error('PYROAdapter','INVALID HEADER DETAILS: ',conn,( hid, ver, hsiz, bsiz,pflags))
 			# try to report error to client, but most likely the connection will terminate:
 			self.returnException(conn, ProtocolError('invalid header'), shutdown=1)
 			return ver,None,pflags
@@ -503,15 +533,29 @@ class PYROAdapter:
 
 		# Unpickle the request, which is a tuple:
 		#  (object ID, method name, flags, (arg1,arg2,...))
+		importer=fromlist=None
 		try:
-			req=self._unpickleRequest(pflags, body)
-			if type(req)!=type(()):
-				raise TypeError("REQUESTDATA ISN'T A TUPLE") # XXX
+			try:
+				import __builtin__
+				importer=agent_import(__builtin__.__import__)
+				__builtin__.__import__=importer
+				req=self._unpickleRequest(pflags, body)
+				if type(req)!=type(()):
+					raise TypeError("REQUESTDATA ISN'T A TUPLE")
+			finally:
+				__builtin__.__import__=importer.orig_import
+
 		except ImportError,x:
 			if Pyro.config.PYRO_MOBILE_CODE:
 				# return a special exception that will be processed by client;
 				# it will call the internal 'remote_supply_code' member
-				self.returnException(conn, _InternalNoModuleError(x.args[0][16:],None),0) # don't shutdown!
+				if importer:
+					modname=importer.name
+					fromlist=importer.fromlist
+				else:
+					modname = x.args[0][16:]
+					fromlist=None
+				self.returnException(conn, _InternalNoModuleError(modname,fromlist),0) # don't shutdown!
 			else:
 				Log.error('PYROAdapter','code problem with incoming object: '+str(x))
 				self.returnException(conn, NoModuleError(* x.args))
@@ -526,53 +570,64 @@ class PYROAdapter:
 			return
 		else:
 			# Do the invocation. We are already running in our own thread.
-			try:
-				flags=req[2]
-				importer=None
-				if not Pyro.config.PYRO_MOBILE_CODE:
-					res = o[0].Pyro_dyncall(req[1],flags,req[3])	# (method,flags,args)
-				else:
-					try:
-						import __builtin__
-						importer=agent_import(__builtin__.__import__)
-						__builtin__.__import__=importer
-						res = o[0].Pyro_dyncall(req[1],flags,req[3])	# (method,flags,args)
-					finally:
-						__builtin__.__import__=importer.orig_import
+			if req[2]&constants.RIF_Oneway and daemon.threaded:   # flags
+				# received a oneway call, run this in its own thread.
+				thread=Thread(target=self._handleInvocation2, args=(daemon,req,pflags,conn,o))
+				thread.setDaemon(1)   # thread must exit at program termination.
+				thread.start()
+			else:
+				# not oneway or not in threaded mode, just do the invocation synchronously
+				self._handleInvocation2(daemon,req,pflags,conn,o)
 
-				if flags&constants.RIF_Oneway:
-					return		# no result, return immediately
-				# reply the result to the caller
-				if pflags&PFLG_XMLPICKLE_GNOSIS:
-					replyflags=PFLG_XMLPICKLE_GNOSIS
-					body=util.getXMLPickle('gnosis').dumps(res,Pyro.config.PYRO_PICKLE_FORMAT)
-				elif pflags&PFLG_XMLPICKLE_PYXML:
-					replyflags=PFLG_XMLPICKLE_PYXML
-					body=util.getXMLPickle('pyxml').dumps(res,Pyro.config.PYRO_PICKLE_FORMAT)
+	def _handleInvocation2(self, daemon, req, pflags, conn, obj):
+		try:
+			flags=req[2]
+			importer=None
+			if not Pyro.config.PYRO_MOBILE_CODE:
+				res = obj[0].Pyro_dyncall(req[1],flags,req[3])	# (method,flags,args)
+			else:
+				try:
+					import __builtin__
+					importer=agent_import(__builtin__.__import__)
+					__builtin__.__import__=importer
+					res = obj[0].Pyro_dyncall(req[1],flags,req[3])	# (method,flags,args)
+				finally:
+					__builtin__.__import__=importer.orig_import
+
+			if flags&constants.RIF_Oneway:
+				return		# no result, return immediately
+			# reply the result to the caller
+			if pflags&PFLG_XMLPICKLE_GNOSIS:
+				replyflags=PFLG_XMLPICKLE_GNOSIS
+				body=util.getXMLPickle('gnosis').dumps(res,Pyro.config.PYRO_PICKLE_FORMAT)
+			elif pflags&PFLG_XMLPICKLE_PYXML:
+				replyflags=PFLG_XMLPICKLE_PYXML
+				body=util.getXMLPickle('pyxml').dumps(res,Pyro.config.PYRO_PICKLE_FORMAT)
+			else:
+				replyflags=0
+				body=pickle.dumps(res,Pyro.config.PYRO_PICKLE_FORMAT)
+			sock_sendmsg(conn.sock, self.createMsg(body,replyflags),self.timeout)
+		except ImportError,ix:
+			if Pyro.config.PYRO_MOBILE_CODE:
+				# Return a special exception that will be processed by client;
+				# it will call the internal 'remote_supply_code' member.
+				# We have to use this seemingly complex way to signal the client
+				# to supply us some code, but it is only a proxy! We can't *call* it!
+				if importer:
+					# grab the import info from our importer
+					name=importer.name
+					fromlist=importer.fromlist
 				else:
-					replyflags=0
-					body=pickle.dumps(res,Pyro.config.PYRO_PICKLE_FORMAT)
-				sock_sendmsg(conn.sock, self.createMsg(body,replyflags),self.timeout)
-			except ImportError,ix:
-				if Pyro.config.PYRO_MOBILE_CODE:
-					# Return a special exception that will be processed by client;
-					# it will call the internal 'remote_supply_code' member.
-					# We have to use this seemingly complex way to signal the client
-					# to supply us some code, but it is only a proxy! We can't *call* it!
-					if importer:
-						# grab the import info from our importer
-						name=importer.name
-						fromlist=importer.fromlist
-					else:
-						name=ix.args[0][16:]
-						fromlist=None
-					Log.msg('PYROAdapter','failed to import',name)
-					self.returnException(conn, _InternalNoModuleError(name,fromlist),0) # don't shutdown!
-				else:
-					Log.error('PYROAdapter','code problem with incoming object: '+str(ix))
-					self.returnException(conn, NoModuleError(* ix.args))
-			except Exception,x:
-				daemon.handleError(conn)
+					# XXX the importerror sometimes doesn't contain the package :-(
+					name=ix.args[0][16:]
+					fromlist=None
+				Log.msg('PYROAdapter','failed to import',name)
+				self.returnException(conn, _InternalNoModuleError(name,fromlist),0) # don't shutdown!
+			else:
+				Log.error('PYROAdapter','code problem with incoming object: '+str(ix))
+				self.returnException(conn, NoModuleError(* ix.args))
+		except Exception:
+			daemon.handleError(conn)
 
 	def returnException(self, conn, exc, shutdown=1, args=None):
 		# return an encapsulated exception to the client
@@ -718,7 +773,7 @@ class TCPConnection:
 
 #-------- The New Connection Validators:
 #-------- DefaultConnValidator checks max number of connections & identification
-#-------- and ident check is done using md5 secure hash of passphrase+challenge.
+#-------- and ident check is done using hmac-md5 secure hash of passphrase+challenge.
 #-------- Contains client- & server-side auth code.
 class DefaultConnValidator:
 	def __init__(self):
@@ -737,11 +792,11 @@ class DefaultConnValidator:
 		Log.warn('DefaultConnValidator','connect authentication failed on conn ',connection)
 		return (0,constants.DENIED_SECURITY)
 	def createAuthToken(self, authid, challenge, peeraddr, URI, daemon):
-		# Called from both client and server, to validate the token.
+		# Called from both client and server, is used to be able to validate the token.
 		# client: URI & peeraddr provided, daemon is None
 		# server: URI is None, peeraddr and daemon provided.
-		# Return secure hash of our authentication phrase & the challenge
-		return md5.md5(authid+challenge).digest()
+		# Return hmac-md5 secure hash of our authentication phrase & the challenge.
+		return hmac.new(challenge, authid).digest()
 	def createAuthChallenge(self, tcpserver, conn):
 		# Server-side only, when new connection comes in.
 		# Challenge is secure hash of: server IP, process ID, timestamp, random value
@@ -752,11 +807,11 @@ class DefaultConnValidator:
 		except:
 			pid=id(self)	# XXX jython has no getpid()
 		string = '%s-%d-%.20f-%.20f' %(str(getIPAddress()), pid, time.time(), random.random())
-		return md5.md5(string).digest()
+		return md5.new(string).digest()
 	def mungeIdent(self, ident):
 		# munge the identification string into something else that's
 		# not easily guessed or recognised, like the md5 hash:
-		return md5.md5(ident).digest()
+		return md5.new(ident).digest()
 	def setAllowedIdentifications(self, ids):
 		if ids is not None:
 			if type(ids) in (types.TupleType, types.ListType):
@@ -882,7 +937,7 @@ class TCPServer:
 							raise ConnectionClosedError()
 						ins,outs,exs=select.select([conn],[],[],2)
 						if conn in ins or conn in exs:
-							self.handleInvocation(conn)			# XXX ugly! calls subclass!
+							self.handleInvocation(conn)
 					except ConnectionClosedError:
 						# client went away. Exit immediately
 						self.removeConnection(conn)
@@ -893,21 +948,21 @@ class TCPServer:
 				# log entry has already been written by newConnValidator
 				self.removeConnection(conn)
 		finally:
-			# print 'EXIT THREAD:',currentThread().getName() # XXX 
+			# print 'EXIT THREAD:',currentThread().getName()
 			self._removeFromConnectionList(None)
 
-	def _removeFromConnectionList(self, object):
-		if self.threaded:
-			object=currentThread()
+	def _removeFromConnectionList(self, obj):
+		if self.threaded and currentThread:
+			obj=currentThread()
 		try:
-			self.connections.remove(object)
+			self.connections.remove(obj)
 		except ValueError:
 			pass
 
 
 	# this is the preferred way of dealing with the request loop.
 	def requestLoop(self, condition=lambda:1, timeout=3, others=[], callback=None):
-		while condition():
+		while condition() and not self.mustShutdown:
 			self.handleRequests(timeout,others,callback)
 
 	def handleRequests(self, timeout=None, others=[], callback=None):
@@ -959,7 +1014,7 @@ class TCPServer:
 			if isinstance(c,TCPConnection):
 				ins.remove(c)
 				try:
-					self.handleInvocation(c)		# XXX ugly! calls subclass!
+					self.handleInvocation(c)
 					if not c.connected:
 						self.removeConnection(c)
 				except ConnectionClosedError:
@@ -972,6 +1027,8 @@ class TCPServer:
 			# the 'others' must have fired...
 			callback(ins)
 
+	# def handleInvocation(self, conn):	.... abstract method (implemented in subclass)
+
 
 	def _handleRequest_Threaded(self,timeout,others,callback):
 		# self.connections is used to keep track of connection Threads
@@ -993,7 +1050,7 @@ class TCPServer:
 					sslsock.accept_ssl()
 				except SSL.SSLError,error:
 					Log.warn('TCPServer','SSL error: '+str(error))
-					print "SSL Error:",error
+					# print "SSL Error:",error
 					csock.close()
 					return
 				csock=sslsock
@@ -1008,12 +1065,11 @@ class TCPServer:
 			self.initTLS(thread.localStorage)
 			self.connections.append(thread)
 			thread.start()
-			# print 'NEW THREAD:'+thread.getName() # XXX
+			# print 'NEW THREAD:'+thread.getName()
 		elif callback:
 			# the 'others' must have fired...
 			callback(ins)
 
-
 	def getLocalStorage(self):
 		# return storage object for this thread.
 		if self.threaded:
