@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 """
-$Id: wxnsc.py,v 1.9 2003/08/26 17:57:35 irmen Exp $
+$Id: wxnsc.py,v 1.10 2005/02/12 23:45:29 irmen Exp $
 
 A wxPython gui to nsc (Pyro Name Server Control tool).
 This gui doesn't have as many features as the xnsc that ships with Pyro,
@@ -31,8 +31,8 @@ Usage (from the commandline):
 """
 
 __author__   = "Jan Finell"
-__date__     = "$Date: 2003/08/26 17:57:35 $"
-__revision__ = "$Revision: 1.9 $"
+__date__     = "$Date: 2005/02/12 23:45:29 $"
+__revision__ = "$Revision: 1.10 $"
 
 #
 # Standard modules
@@ -215,6 +215,7 @@ class wxNSC(wxFrame):
       self.buttonDeleteSelected.Enable(enable)
       self.buttonRegisterItem.Enable(enable)
       self.buttonShowMeta.Enable(enable)
+      self.buttonSetMeta.Enable(enable)
 
    def update(self):
       """
@@ -391,6 +392,20 @@ class wxNSC(wxFrame):
          self._logError('unable to register, error: %s' % e)
       return 0
 
+   def nsc_set_meta(self, name, meta):
+      """
+      Set user meta data 
+      """
+      try:
+         self.NS.setMeta(name, meta)
+         self._log('set user meta data on '+name)
+         return 1
+      except NamingError, e:
+         self._logError('unable to set user meta data,\nName Server error: %s' % e)
+      except Exception, e:
+         self._logError('unable to set user meta data, error: %s' % e)
+      return 0
+
    def nsc_show_meta(self, name):
       fullName = self.NS.fullName(name)
       try:
@@ -445,6 +460,23 @@ class wxNSC(wxFrame):
             tree.SetItemImage(groupI, self.__idGroupOpen,
                               wxTreeItemIcon_Expanded)
             tree.Expand(parentGroupI)
+
+   def OnSetMeta(self, ev=None):
+      """
+      set user meta on selected groups + items
+      """
+      tree = self.treeCtrlItems
+      itemsL, groupsL = self._getSelections()
+      namesL  = itemsL + groupsL
+      if namesL:     
+         namesS = ',\n '.join(namesL)
+         msg = 'User meta data string for:\n %s' % namesS
+         dlg = wxTextEntryDialog(self, msg, 'Enter meta data')
+         dlg.CentreOnParent()
+         if dlg.ShowModal() == wxID_OK:
+            meta=dlg.GetValue()
+            for name in namesL:
+               self.nsc_set_meta(name,meta)
       
    def OnDelete(self, ev=None):
       """
@@ -514,6 +546,7 @@ class wxNSC(wxFrame):
       if self._checkNS():
          self.update()
       self.nsc_ping()
+
       
    def OnRegisterItem(self, event):
       """
@@ -661,6 +694,7 @@ class wxNSC(wxFrame):
       EVT_BUTTON(self, self.buttonDeleteSelected.GetId(), self.OnDelete)
       EVT_BUTTON(self, self.buttonRegisterItem.GetId(), self.OnRegisterItem)
       EVT_BUTTON(self, self.buttonShowMeta.GetId(), self.OnShowMeta)
+      EVT_BUTTON(self, self.buttonSetMeta.GetId(), self.OnSetMeta)
       EVT_TEXT_ENTER(self, self.txtCtrlNSHost.GetId(), self.OnCheckNS)
       EVT_TEXT_ENTER(self, self.txtCtrlNSPort.GetId(), self.OnCheckNS)
       EVT_CHAR(self.treeCtrlItems, self.OnKeyPressed)
@@ -671,7 +705,7 @@ class wxNSC(wxFrame):
       """
       parent = wxPanel(self, -1)
       sizer0 = wxBoxSizer(wxVERTICAL)
-      sizer0.AddSizer(self._buildTopBar(parent), 0, wxALIGN_LEFT|wxGROW, 5)
+      sizer0.Add(self._buildTopBar(parent), 0, wxALIGN_LEFT|wxGROW, 5)
 
       splitter = wxSplitterWindow(parent, -1)
       #- TOP PART --------------------------------------------------------#
@@ -679,7 +713,7 @@ class wxNSC(wxFrame):
       topSizer = wxBoxSizer(wxVERTICAL)
       self.treeCtrlItems = wxTreeCtrl(topParent, -1,
                          style = wxTR_TWIST_BUTTONS|wxTR_LINES_AT_ROOT|wxTR_HAS_BUTTONS|wxTR_HIDE_ROOT|wxTR_MULTIPLE)
-      topSizer.AddWindow(self.treeCtrlItems, 1, wxEXPAND, 5)
+      topSizer.Add(self.treeCtrlItems, 1, wxEXPAND, 5)
       topParent.SetAutoLayout( true )
       topParent.SetSizer(topSizer )
       topSizer.Fit(topParent)
@@ -691,17 +725,17 @@ class wxNSC(wxFrame):
       self.txtCtrlLog=wxStdoutLog(bottomParent, -1, "",
                                   size= wxSize(-1, 10),
                                   style=wxTE_MULTILINE|wxTE_READONLY|wxTE_RICH)
-      bottomSizer.AddWindow(self.txtCtrlLog, 1, wxEXPAND, 5)
+      bottomSizer.Add(self.txtCtrlLog, 1, wxEXPAND, 5)
       bottomParent.SetAutoLayout( true )
       bottomParent.SetSizer(bottomSizer )
       bottomSizer.Fit(bottomParent)
       bottomSizer.SetSizeHints(bottomParent)
       #-------------------------------------------------------------------#
       splitter.SplitHorizontally(topParent,bottomParent, -100)
-      sizer0.AddWindow(splitter, 1, wxEXPAND|wxALIGN_CENTRE, 5)
+      sizer0.Add(splitter, 1, wxEXPAND|wxALIGN_CENTRE, 5)
       
       self.buttonClose = wxButton(parent, -1, 'Close')         # buttonClose
-      sizer0.AddWindow(self.buttonClose, 0, wxALIGN_CENTRE|wxALL, 5)
+      sizer0.Add(self.buttonClose, 0, wxALIGN_CENTRE|wxALL, 5)
       
       parent.SetAutoLayout( true )
       parent.SetSizer( sizer0 )
@@ -717,52 +751,55 @@ class wxNSC(wxFrame):
       sizer1 = wxBoxSizer(wxHORIZONTAL)
       txt1 = wxStaticText(parent, -1, 'Name Server:')
       txt1.SetForegroundColour(wxBLUE)
-      sizer1.AddWindow(txt1, 0, wxALIGN_LEFT|wxALIGN_CENTRE|wxALL, 5)
+      sizer1.Add(txt1, 0, wxALIGN_LEFT|wxALIGN_CENTRE|wxALL, 5)
       self.txtCtrlNSHost = wxTextCtrl(parent, -1, '', size=wxSize(300,-1),
                                       style=wxTE_PROCESS_ENTER)
-      sizer1.AddWindow(self.txtCtrlNSHost, 0,
+      sizer1.Add(self.txtCtrlNSHost, 0,
                        wxALIGN_LEFT|wxALIGN_BOTTOM|wxTOP|wxBOTTOM, 5)
       txtColon = wxStaticText(parent, -1, ':')
       txtColon.SetForegroundColour(wxBLUE)
-      sizer1.AddWindow(txtColon, 0,
+      sizer1.Add(txtColon, 0,
                        wxALIGN_LEFT|wxALIGN_CENTRE|wxTOP|wxBOTTOM, 5)
       self.txtCtrlNSPort = wxTextCtrl(parent, -1, '', size=wxSize(50,-1),
                                       style=wxTE_PROCESS_ENTER)
-      sizer1.AddWindow(self.txtCtrlNSPort, 0,
+      sizer1.Add(self.txtCtrlNSPort, 0,
                        wxALIGN_LEFT|wxALIGN_BOTTOM|wxTOP|wxBOTTOM, 5)
       self.buttonUpdate = wxButton(parent, -1, 'Update')       # buttonUpdate
-      sizer1.AddWindow(self.buttonUpdate, 0, wxALIGN_LEFT|wxALL, 5)
+      sizer1.Add(self.buttonUpdate, 0, wxALIGN_LEFT|wxALL, 5)
       self.buttonPing = wxButton(parent, -1, 'Ping')           # buttonPing
-      sizer1.AddWindow(self.buttonPing, 0, wxALIGN_LEFT|wxALL, 5)
-      sizer0.AddSizer(sizer1, 0, wxALIGN_LEFT|wxGROW, 5)
+      sizer1.Add(self.buttonPing, 0, wxALIGN_LEFT|wxALL, 5)
+      sizer0.Add(sizer1, 0, wxALIGN_LEFT|wxGROW, 5)
       #--
       lineH1 = wxStaticLine(parent, -1, style=wxLI_HORIZONTAL)
-      sizer0.AddWindow(lineH1, 0, wxGROW|wxALIGN_CENTRE|wxLEFT|wxRIGHT, 5)
+      sizer0.Add(lineH1, 0, wxGROW|wxALIGN_CENTRE|wxLEFT|wxRIGHT, 5)
       #--
       sizer2 = wxBoxSizer(wxHORIZONTAL)
       self.buttonDeleteGroup = wxButton(parent, -1, ' Delete group(s) ')
-      sizer2.AddWindow(self.buttonDeleteGroup, 0,
+      sizer2.Add(self.buttonDeleteGroup, 0,
                        wxALIGN_LEFT|wxALIGN_CENTER|wxTOP|wxLEFT|wxBOTTOM, 5)
       self.buttonCreateGroup = wxButton(parent, -1, ' Create group... ')
-      sizer2.AddWindow(self.buttonCreateGroup, 0,
+      sizer2.Add(self.buttonCreateGroup, 0,
                        wxALIGN_LEFT|wxALIGN_CENTER|wxTOP|wxLEFT|wxBOTTOM, 5)
       lineV1 = wxStaticLine(parent, -1, style=wxLI_VERTICAL)
-      sizer2.AddWindow(lineV1, 0,
+      sizer2.Add(lineV1, 0,
                        wxALL|wxGROW, 5)
       self.buttonDeleteSelected = wxButton(parent, -1,
                                            ' Delete item(s) ')
-      sizer2.AddWindow(self.buttonDeleteSelected, 0,
+      sizer2.Add(self.buttonDeleteSelected, 0,
                        wxALIGN_LEFT|wxALIGN_CENTER|wxALL, 5)
       self.buttonRegisterItem = wxButton(parent, -1, ' Register item... ')
-      sizer2.AddWindow(self.buttonRegisterItem, 0,
+      sizer2.Add(self.buttonRegisterItem, 0,
                        wxALIGN_LEFT|wxALIGN_CENTER|wxTOP|wxBOTTOM, 5)
       lineV2 = wxStaticLine(parent, -1, style=wxLI_VERTICAL)
-      sizer2.AddWindow(lineV2, 0,
+      sizer2.Add(lineV2, 0,
                        wxALL|wxGROW, 5)
       self.buttonShowMeta = wxButton(parent, -1, ' Show meta ')
-      sizer2.AddWindow(self.buttonShowMeta, 0,
+      sizer2.Add(self.buttonShowMeta, 0,
                        wxALIGN_LEFT|wxALIGN_CENTER|wxTOP|wxBOTTOM, 5)
-      sizer0.AddSizer(sizer2, 0, wxALIGN_LEFT, 5)
+      self.buttonSetMeta = wxButton(parent, -1, ' Set meta ')
+      sizer2.Add(self.buttonSetMeta, 0,
+                       wxALIGN_LEFT|wxALIGN_CENTER|wxTOP|wxBOTTOM, 5)
+      sizer0.Add(sizer2, 0, wxALIGN_LEFT, 5)
       #--
       return sizer0
 
@@ -807,24 +844,24 @@ class WinMessageDialog(wxDialog):
         if artID:
             bmp = wxArtProvider_GetBitmap(artID, wxART_MESSAGE_BOX, (48,48))
             bmpIcon = wxStaticBitmap(parent, -1, bmp)
-            msgSizer.AddWindow(bmpIcon, 0, wxALIGN_CENTRE|wxALL, 5)
+            msgSizer.Add(bmpIcon, 0, wxALIGN_CENTRE|wxALL, 5)
 
         # msg #
         txtMsg = wxStaticText(parent, -1, msg, style=wxALIGN_CENTRE)
-        msgSizer.AddWindow(txtMsg, 0, wxALIGN_CENTRE|wxALL, 5)
+        msgSizer.Add(txtMsg, 0, wxALIGN_CENTRE|wxALL, 5)
 
-        sizer.AddSizer(msgSizer, 0, wxALIGN_CENTRE, 5)
+        sizer.Add(msgSizer, 0, wxALIGN_CENTRE, 5)
         line = wxStaticLine(parent, -1, style=wxLI_HORIZONTAL)
-        sizer.AddWindow(line, 0, wxGROW|wxALL, 5)
+        sizer.Add(line, 0, wxGROW|wxALL, 5)
         #-- buttons --#
         btnSizer = wxBoxSizer(wxHORIZONTAL)
 
         if style & wxYES_NO == wxYES_NO:
             btnYes = wxButton(parent, wxID_YES, 'Yes')
-            btnSizer.AddWindow(btnYes, 0,
+            btnSizer.Add(btnYes, 0,
                                wxALIGN_CENTRE|wxLEFT|wxRIGHT, 10)
             btnNo  = wxButton(parent, wxID_NO, 'No')
-            btnSizer.AddWindow(btnNo, 0,
+            btnSizer.Add(btnNo, 0,
                                wxALIGN_CENTRE|wxLEFT|wxRIGHT, 10)
             if style & wxYES_DEFAULT == wxYES_DEFAULT:
                 btnYes.SetDefault()
@@ -836,14 +873,14 @@ class WinMessageDialog(wxDialog):
             if style & wxOK == wxOK:
                 btnOK = wxButton(parent, wxID_OK, 'OK')
                 btnOK.SetDefault()
-                btnSizer.AddWindow(btnOK, 0,
+                btnSizer.Add(btnOK, 0,
                                    wxALIGN_CENTRE|wxLEFT|wxRIGHT, 10)
             if style & wxCANCEL == wxCANCEL:
                 btnCancel = wxButton(parent, wxID_CANCEL, 'Cancel')
-                btnSizer.AddWindow(btnCancel, 0,
+                btnSizer.Add(btnCancel, 0,
                                    wxALIGN_CENTRE|wxLEFT|wxRIGHT, 10)
 
-        sizer.AddSizer(btnSizer, 0, wxALIGN_CENTRE|wxTOP, 5)
+        sizer.Add(btnSizer, 0, wxALIGN_CENTRE|wxTOP, 5)
         #--
         parent.SetAutoLayout( true )
         parent.SetSizer(sizer )
