@@ -12,8 +12,8 @@ import agent.ShoppingAgent as SA
 
 Pyro.config.PYRO_MOBILE_CODE=1		# Enable mobile code
 
-# Get a proxy with attrs (static or DynamicProxyWithAttrs)
-mall = testclient.start('shop_proxy','Mall','ShoppingMall',1)
+# Get a proxy with attrs
+mall = testclient.getproxy('ShoppingMall',True)
 
 import sys, os, string
 import Pyro.core
