@@ -6,8 +6,8 @@ sys.path.insert(0,os.pardir)	# to find testclient.py
 
 import testclient
 
-# Get a proxy with attrs (static or DynamicProxyWithAttrs)
-test = testclient.start('attrs_proxy','testclass','attributes',1)
+# Get a proxy with attrs 
+test = testclient.getproxy('attributes',True)
 
 name = raw_input('Enter a name: ')
 
