@@ -25,7 +25,7 @@ daemon.useNameServer(ns)
 daemon.setAllowedIdentifications(['s3cr3t'])
 
 # connect new instance, but using persistent mode
-daemon.connectPersistent(testobject(),'authentication')
+daemon.connectPersistent(testobject(),':test.authentication')
 
 # enter the service loop.
 print 'Server started.'
