@@ -25,7 +25,7 @@ ns = locator.getNS()
 daemon.useNameServer(ns)
 
 # connect new instance, but using persistent mode
-daemon.connectPersistent(testobject(),'autoreconnect')
+daemon.connectPersistent(testobject(),':test.autoreconnect')
 
 # enter the service loop.
 print 'Server started.'
