@@ -7,7 +7,7 @@ sys.path.insert(0,os.pardir)		# to find testclient.py
 import testclient
 import bench
 
-object = testclient.start('bench_proxy','bench','benchmark')
+object = testclient.getproxy('benchmark')
 object._setOneway('oneway')
 object._setTimeout(4)
 
@@ -25,8 +25,9 @@ def fb(): void=object.meth3('stringetje')
 def fc(): void=object.meth4('stringetje')
 def fd(): void=object.bigarg('Argument'*50)
 def fe(): void=object.oneway('stringetje',432423434)
+def ff(): void=object.mapping( {"aap":42, "noot": 99, "mies": 987654} )
 
-funcs = (f1,f2,f3,f4,f5,f6,f7,f8,f9,fa,fb,fc,fd,fe)
+funcs = (f1,f2,f3,f4,f5,f6,f7,f8,f9,fa,fb,fc,fd,fe,ff)
 import time
 
 print '-------- BENCHMARK REMOTE OBJECT ---------'
@@ -61,3 +62,4 @@ print 'total method calls',len(funcs)*iters
 avg_normal_msec = 1000.0*duration/(len(funcs)*iters)
 print 'avg. time per method call: %.4f' % avg_normal_msec,"msec"
 print 'Normal method call is %.4f times faster than Pyro method call.'%(avg_pyro_msec/avg_normal_msec)
+
