@@ -11,7 +11,7 @@ class Chain(Pyro.core.ObjBase):
 		self.next=None
 	def process(self,message):
 		if self.next is None:
-			self.next=Pyro.core.getProxyForURI("PYRONAME://:Default.chain_"+self.nextName)
+			self.next=Pyro.core.getProxyForURI("PYRONAME://:test.chain_"+self.nextName)
 		if self.name in message:
 			print "Back at",self.name,"; we completed the circle!"
 			return ["complete at "+self.name]
