@@ -12,7 +12,7 @@ daemon.useNameServer(ns)
 objName='C'
 nextName='A'
 
-daemon.connect(chain.Chain(objName,nextName),'chain_'+objName)
+daemon.connect(chain.Chain(objName,nextName),':test.chain_'+objName)
 
 # enter the service loop.
 print 'Server started obj',objName
