@@ -8,7 +8,7 @@ import Pyro.util
 
 from excep import MyError
 
-test = testclient.start('excep_proxy', 'testclass', 'exceptions')
+test = testclient.getproxy('exceptions')
 
 print test.div(2.0,9.0)
 try:
