@@ -5,7 +5,7 @@ sys.path.insert(0, os.pardir)	# to find testclient.py
 
 import testclient
 
-fact = testclient.start('factory_proxy','CarFactory','factory')
+fact = testclient.getproxy('factory')
 
 print 'Local Python PID =', os.getpid()
 
