@@ -5,7 +5,7 @@ import Pyro.core
 
 Pyro.core.initClient()
 
-test = Pyro.core.getProxyForURI("PYRONAME://simple")
+test = Pyro.core.getProxyForURI("PYRONAME://:test.simple")
 
 print test.mul(111,9)
 print test.add(100,222)
